#define G_LOG_DOMAIN "gpod-channel"

#include "gpod-channel.h"

#include <glib.h>

/**
 * SECTION:gpod-channel
 * @short_description: a container to hold a RSS channel in a #GpodFeed
 *
 * #GpodChannel is the representation of a RSS channel in a valid RSS feed. It
 * contains information about the channel like title, link, publish date, but
 * more importantly all the RSS items; which are stored as #GpodItems.
 *
 * Since #GpodChannel can contain many #GpodItems it can be expensive and for
 * this reason they are reference counted. You can control the lifetime of a
 * #GpodChannel using gpod_channel_ref() and gpod_channel_unref().
 */

G_DEFINE_BOXED_TYPE (GpodChannel, gpod_channel, gpod_channel_ref, gpod_channel_unref)

/**
 * gpod_channel_new: (constructor)
 *
 * Creates a new #GpodChannel, a container to hold a RSS channel in a #GpodFeed.
 *
 * Returns: (transfer full): the newly created #GpodChannel
 */
GpodChannel *
gpod_channel_new (void)
{
    GpodChannel *object;

    object = g_slice_new0 (GpodChannel);
    object->ref_count = 1;

    object->title = NULL;
    object->description = NULL;
    object->link = NULL;
    object->publish_date = NULL;
    object->last_build_date = NULL;
    object->image = NULL;

    object->items = g_ptr_array_new ();

    return object;
}

/**
 * gpod_channel_ref:
 * @object: a #GpodChannel
 *
 * Increase by one the reference count of a #GpodChannel.
 *
 * Returns: (transfer none): the passed #GpodChannel, with the reference count
 *                           increased by one.
 */
GpodChannel *
gpod_channel_ref (GpodChannel *object)
{
    g_return_val_if_fail (object != NULL, NULL);
    g_return_val_if_fail (object->ref_count > 0, NULL);

    object->ref_count++;

    return object;
}

/**
 * gpod_channel_unref:
 * @object: a #GpodChannel
 *
 * Decreases by one the reference count of a #GpodChannel. If the
 * reference count reaches zero, the object is destroyed and all
 * its allocated resources are freed.
 */
void
gpod_channel_unref (GpodChannel *object)
{
    g_return_if_fail (object != NULL);
    g_return_if_fail (object->ref_count > 0);

    if (--object->ref_count == 0)
    {
        guint i;

        if (object->title != NULL)
            g_free (object->title);

        if (object->description != NULL)
            g_free (object->description);

        if (object->link != NULL)
            g_free (object->link);

        if (object->publish_date != NULL)
            g_free (object->publish_date);

        if (object->last_build_date != NULL)
            g_free (object->last_build_date);

        if (object->image != NULL)
            g_free (object->image);

        for (i = 0; i < object->items->len; i++)
            gpod_item_unref (g_ptr_array_index (object->items, i));

        g_ptr_array_free (object->items, TRUE);

        object->title = NULL;
        object->description = NULL;
        object->link = NULL;
        object->publish_date = NULL;
        object->last_build_date = NULL;
        object->image = NULL;
        object->items = NULL;

        g_slice_free (GpodChannel, object);
    }
}
